@@ -292,14 +292,19 @@ bool config_to_color(const char* text, argb_t* color)
 {
     ssize_t num;
 
-    if (*text == '#') {
+    if (*text == '#' || isspace(*text)) {
         ++text;
     }
 
-    if (str_to_num(text, 0, &num, 16) && num >= 0 && num <= 0xffffffff) {
-        *color = num;
-        return true;
+    if (!str_to_num(text, 0, &num, 16) || num < 0 || num > 0xffffffff) {
+        return false;
     }
 
-    return false;
+    if (strlen(text) > 6) { // value with alpha (RRGGBBAA)
+        *color = (num >> 8) | ARGB_SET_A(num);
+    } else {
+        *color = num | ARGB_SET_A(0xff);
+    }
+
+    return true;
 }
