@@ -2,7 +2,7 @@
 // PNG format decoder.
 // Copyright (C) 2020 Artem Senichev <artemsen@gmail.com>
 
-#include "loader.h"
+#include "../loader.h"
 
 #include <png.h>
 #include <setjmp.h>
@@ -143,27 +143,17 @@ static bool decode_frame(struct image* ctx, png_struct* png, png_info* info,
             dispose = PNG_DISPOSE_OP_BACKGROUND;
         } else if (index + 1 < ctx->num_frames) {
             struct pixmap* next = &ctx->frames[index + 1].pm;
-            pixmap_copy(next, 0, 0, &frame->pm, frame->pm.width,
-                        frame->pm.height);
+            pixmap_copy(&frame->pm, next, 0, 0, false);
         }
     }
 
     // put frame on final pixmap
-    switch (blend) {
-        case PNG_BLEND_OP_SOURCE:
-            pixmap_copy(&frame->pm, x, y, &frame_pm, frame_pm.width,
-                        frame_pm.height);
-            break;
-        case PNG_BLEND_OP_OVER:
-            pixmap_over(&frame->pm, x, y, &frame_pm, frame_pm.width,
-                        frame_pm.height);
-            break;
-    }
+    pixmap_copy(&frame_pm, &frame->pm, x, y, blend == PNG_BLEND_OP_OVER);
 
     // handle dispose
     if (dispose == PNG_DISPOSE_OP_NONE && index + 1 < ctx->num_frames) {
         struct pixmap* next = &ctx->frames[index + 1].pm;
-        pixmap_copy(next, 0, 0, &frame->pm, frame->pm.width, frame->pm.height);
+        pixmap_copy(&frame->pm, next, 0, 0, false);
     }
 
     rc = true;
@@ -247,12 +237,10 @@ enum loader_status decode_png(struct image* ctx, const uint8_t* data,
     // create decoder
     png = png_create_read_struct(PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
     if (!png) {
-        image_print_error(ctx, "unable to initialize png decoder");
         return ldr_fmterror;
     }
     info = png_create_info_struct(png);
     if (!info) {
-        image_print_error(ctx, "unable to create png object");
         png_destroy_read_struct(&png, NULL, NULL);
         return ldr_fmterror;
     }
@@ -260,7 +248,6 @@ enum loader_status decode_png(struct image* ctx, const uint8_t* data,
     // setup error handling
     if (setjmp(png_jmpbuf(png))) {
         png_destroy_read_struct(&png, &info, NULL);
-        image_print_error(ctx, "failed to decode png");
         return ldr_fmterror;
     }
 
