@@ -4,9 +4,9 @@
 
 #include "info.h"
 
-#include "canvas.h"
 #include "config.h"
 #include "imagelist.h"
+#include "loader.h"
 #include "str.h"
 
 #include <stdarg.h>
@@ -108,6 +108,7 @@ struct info_block {
 /** Info data context. */
 struct info_context {
     enum info_mode mode;
+    int timeout;
     const char* file;
     struct info_line* exif_lines;
     size_t exif_num;
@@ -208,6 +209,26 @@ static enum config_status load_config(const char* key, const char* value)
     size_t scheme_sz = 0;
     ssize_t index;
 
+    if (strcmp(key, "timeout") == 0) {
+        ssize_t num, num_max;
+        bool is_percent;
+        size_t len = strlen(value);
+
+        if (len == 0) {
+            return cfgst_invalid_value;
+        }
+        is_percent = (value[len - 1] == '%');
+        if (is_percent) {
+            --len;
+        }
+        num_max = is_percent ? 100 : 86400;
+        if (!str_to_num(value, len, &num, 0) && num >= 0 && num <= num_max) {
+            return cfgst_invalid_value;
+        }
+        ctx.timeout = num * (is_percent ? -1 : 1);
+        return cfgst_ok;
+    }
+
     if (strcmp(key, "mode") == 0) {
         index = str_index(mode_names, value, 0);
         if (index < 0) {
@@ -275,6 +296,7 @@ void info_create(void)
 {
     // set defaults
     ctx.mode = info_mode_full;
+    ctx.timeout = 0;
     ctx.frame = UINT32_MAX;
     ctx.index = UINT32_MAX;
     SET_DEFAULT(info_mode_full, info_top_left, default_full_top_left);
@@ -340,18 +362,17 @@ void info_set_mode(const char* mode)
     }
 }
 
-void info_update(size_t frame_idx)
+void info_update(size_t frame_idx, float scale)
 {
-    const struct image_entry entry = image_list_current();
-    const struct image* image = entry.image;
+    const struct image* image = loader_current_image();
     char buffer[64];
 
-    if (ctx.file != image->file_path) {
+    if (ctx.file != image->source) {
         if (is_visible(info_file_name)) {
-            update_field(image->file_name, &ctx.fields[info_file_name].value);
+            update_field(image->name, &ctx.fields[info_file_name].value);
         }
         if (is_visible(info_file_path)) {
-            update_field(image->file_path, &ctx.fields[info_file_path].value);
+            update_field(image->source, &ctx.fields[info_file_path].value);
         }
         if (is_visible(info_file_size)) {
             const size_t mib = 1024 * 1024;
@@ -369,7 +390,7 @@ void info_update(size_t frame_idx)
         }
 
         ctx.frame = UINT32_MAX; // force refresh frame info
-        ctx.file = image->file_path;
+        ctx.file = image->source;
     }
 
     if (is_visible(info_frame) &&
@@ -381,15 +402,15 @@ void info_update(size_t frame_idx)
         update_field(buffer, &ctx.fields[info_frame].value);
     }
 
-    if (is_visible(info_index) && ctx.index != entry.index) {
-        ctx.index = entry.index;
+    if (is_visible(info_index) && ctx.index != loader_current_index()) {
+        ctx.index = loader_current_index();
         snprintf(buffer, sizeof(buffer), "%lu of %lu", ctx.index + 1,
                  image_list_size());
         update_field(buffer, &ctx.fields[info_index].value);
     }
 
     if (is_visible(info_scale)) {
-        const size_t scale_percent = canvas_get_scale() * 100;
+        const size_t scale_percent = scale * 100;
         if (ctx.scale != scale_percent) {
             ctx.scale = scale_percent;
             snprintf(buffer, sizeof(buffer), "%ld%%", ctx.scale);
@@ -531,3 +552,8 @@ const struct info_line* info_lines(enum info_position pos)
 
     return block->lines;
 }
+
+int info_timeout(void)
+{
+    return ctx.timeout;
+}
