@@ -45,8 +45,9 @@ void info_set_mode(const char* mode);
 /**
  * Refresh info data.
  * @param frame_idx index of the current frame
+ * @param scale current scale factor
  */
-void info_update(size_t frame_idx);
+void info_update(size_t frame_idx, float scale);
 
 /**
  * Set status text.
@@ -68,3 +69,10 @@ size_t info_height(enum info_position pos);
  * @return pointer to the lines array
  */
 const struct info_line* info_lines(enum info_position pos);
+
+/**
+ * Get info display timeout.
+ * @return 0 if timeout disabled, positive number for absolute time in second,
+ * or negative number for slideshow relative percents.
+ */
+int info_timeout(void);
