@@ -77,6 +77,7 @@ struct ui {
 
     // cross-desktop
     struct xdg {
+        bool initialized;
         struct xdg_wm_base* base;
         struct xdg_surface* surface;
         struct xdg_toplevel* toplevel;
@@ -223,7 +224,8 @@ static bool recreate_buffers(void)
     }
 
     ctx.wnd.current = ctx.wnd.buffer0;
-    viewer_on_resize(ctx.wnd.width, ctx.wnd.height, ctx.wnd.scale);
+    viewer_on_resize();
+
     return true;
 }
 
@@ -329,7 +331,7 @@ static void on_pointer_motion(void* data, struct wl_pointer* wl_pointer,
     if (ctx.mouse.active) {
         const int dx = x - ctx.mouse.x;
         const int dy = y - ctx.mouse.y;
-        if (dx && dy) {
+        if (dx || dy) {
             viewer_on_drag(dx, dy);
         }
     }
@@ -412,12 +414,12 @@ static void on_xdg_surface_configure(void* data, struct xdg_surface* surface,
 {
     xdg_surface_ack_configure(surface, serial);
 
-    if (!ctx.wnd.current && !recreate_buffers()) {
-        ctx.state = state_error;
-        return;
+    if (ctx.xdg.initialized) {
+        ui_redraw();
+    } else {
+        wl_surface_attach(ctx.wl.surface, ctx.wnd.current, 0, 0);
+        wl_surface_commit(ctx.wl.surface);
     }
-
-    ui_redraw();
 }
 
 static const struct xdg_surface_listener xdg_surface_listener = {
@@ -437,15 +439,22 @@ static void handle_xdg_toplevel_configure(void* data, struct xdg_toplevel* lvl,
                                           int32_t width, int32_t height,
                                           struct wl_array* states)
 {
-    const int32_t cur_width = (int32_t)(ctx.wnd.width / ctx.wnd.scale);
-    const int32_t cur_height = (int32_t)(ctx.wnd.height / ctx.wnd.scale);
+    bool reset_buffers = (ctx.wnd.current == NULL);
 
-    if (width && height && (width != cur_width || height != cur_height)) {
-        ctx.wnd.width = width * ctx.wnd.scale;
-        ctx.wnd.height = height * ctx.wnd.scale;
-        if (!recreate_buffers()) {
-            ctx.state = state_error;
+    if (width && height) {
+        const size_t new_width = width * ctx.wnd.scale;
+        const size_t new_height = height * ctx.wnd.scale;
+        if (width != (int32_t)ctx.wnd.width ||
+            height != (int32_t)ctx.wnd.height) {
+            ctx.wnd.width = new_width;
+            ctx.wnd.height = new_height;
+            reset_buffers = true;
         }
+        ctx.xdg.initialized = true;
+    }
+
+    if (reset_buffers && !recreate_buffers()) {
+        ctx.state = state_error;
     }
 }
 
@@ -682,7 +691,7 @@ static enum config_status load_config(const char* key, const char* value)
     return status;
 }
 
-void ui_init(void)
+void ui_create(void)
 {
     struct timespec ts;
 
@@ -813,9 +822,7 @@ bool ui_run(void)
             const ssize_t sz = sizeof(repeats);
             if (read(ctx.repeat.fd, &repeats, sz) == sz) {
                 const uint8_t mods = keybind_mods(ctx.xkb.state);
-                while (repeats--) {
-                    viewer_on_keyboard(ctx.repeat.key, mods);
-                }
+                viewer_on_keyboard(ctx.repeat.key, mods);
             }
         }
 
@@ -839,7 +846,7 @@ void ui_stop(void)
 
 void ui_redraw(void)
 {
-    struct pixmap wnd = PIXMAP_ATTACH(NULL, ctx.wnd.width, ctx.wnd.height);
+    struct pixmap wnd;
 
     if (!ctx.wnd.current) {
         return; // not yet initialized
@@ -853,6 +860,8 @@ void ui_redraw(void)
     }
 
     // draw to window buffer
+    wnd.width = ctx.wnd.width;
+    wnd.height = ctx.wnd.height;
     wnd.data = wl_buffer_get_user_data(ctx.wnd.current);
     viewer_on_redraw(&wnd);
 
@@ -913,6 +922,11 @@ size_t ui_get_height(void)
     return ctx.wnd.height;
 }
 
+size_t ui_get_scale(void)
+{
+    return ctx.wnd.scale;
+}
+
 void ui_toggle_fullscreen(void)
 {
     ctx.fullscreen = !ctx.fullscreen;
