@@ -8,9 +8,19 @@
 #include "pixmap.h"
 
 // Configuration parameters
+#define VIEWER_CFG_SCALE          "scale"
+#define VIEWER_CFG_ANTIALIASING   "antialiasing"
+#define VIEWER_CFG_FIXED          "fixed"
+#define VIEWER_CFG_TRANSPARENCY   "transparency"
+#define VIEWER_CFG_BACKGROUND     "background"
 #define VIEWER_CFG_SLIDESHOW      "slideshow"
 #define VIEWER_CFG_SLIDESHOW_TIME "slideshow_time"
 
+/**
+ * Create viewer context.
+ */
+void viewer_create(void);
+
 /**
  * Initialize viewer context.
  */
@@ -24,7 +34,7 @@ void viewer_free(void);
 /**
  * Reset state: reload image file, set initial scale etc.
  */
-void viewer_reset(void);
+void viewer_reload(void);
 
 /**
  * Redraw handler.
@@ -34,20 +44,18 @@ void viewer_on_redraw(struct pixmap* window);
 
 /**
  * Window resize handler.
- * @param width,height new window size
- * @param scale window scale factor
  */
-void viewer_on_resize(size_t width, size_t height, size_t scale);
+void viewer_on_resize(void);
 
 /**
  * Key press handler.
  * @param key code of key pressed
- * @param mods key modifires (ctrl/alt/shift)
+ * @param mods key modifiers (ctrl/alt/shift)
  */
 void viewer_on_keyboard(xkb_keysym_t key, uint8_t mods);
 
 /**
- * Image drap handler.
+ * Image drag handler.
  * @param dx,dy delta to move viewpoint
  */
 void viewer_on_drag(int dx, int dy);
